@@ -1,29 +1,31 @@
 #!/bin/bash
 # Copyright 1999-2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: ebuild.sh 8231 2007-10-22 20:33:08Z zmedico $
+# $Id: ebuild.sh 9196 2008-01-12 01:47:04Z zmedico $
 
 PORTAGE_BIN_PATH="${PORTAGE_BIN_PATH:-/usr/lib/portage/bin}"
 PORTAGE_PYM_PATH="${PORTAGE_PYM_PATH:-/usr/lib/portage/pym}"
-declare -rx PORTAGE_BIN_PATH PORTAGE_PYM_PATH
 
 SANDBOX_PREDICT="${SANDBOX_PREDICT}:/proc/self/maps:/dev/console:/dev/random"
 export SANDBOX_PREDICT="${SANDBOX_PREDICT}:${PORTAGE_PYM_PATH}:${PORTAGE_DEPCACHEDIR}"
 export SANDBOX_WRITE="${SANDBOX_WRITE}:/dev/shm:/dev/stdout:/dev/stderr:${PORTAGE_TMPDIR}"
-export SANDBOX_READ="${SANDBOX_READ}:/dev/shm:/dev/stdin:${PORTAGE_TMPDIR}"
+export SANDBOX_READ="${SANDBOX_READ}:/:/dev/shm:/dev/stdin:${PORTAGE_TMPDIR}"
+# Don't use sandbox's BASH_ENV for new shells because it does
+# 'source /etc/profile' which can interfere with the build
+# environment by modifying our PATH.
+unset BASH_ENV
+
+# sandbox's bashrc sources /etc/profile which unsets ROOTPATH,
+# so we have to back it up and restore it.
+if [ -n "${PORTAGE_ROOTPATH}" ] ; then
+	export ROOTPATH=${PORTAGE_ROOTPATH}
+	unset PORTAGE_ROOTPATH
+fi
 
 if [ ! -z "${PORTAGE_GPG_DIR}" ]; then
 	SANDBOX_PREDICT="${SANDBOX_PREDICT}:${PORTAGE_GPG_DIR}"
 fi
 
-declare -rx EBUILD_PHASE
-
-if [ "$*" != "depend" ] && [ "$*" != "clean" ] && [ "$*" != "nofetch" ]; then
-	if [ -f "${T}/environment" ]; then
-		source "${T}/environment" >& /dev/null
-	fi
-fi
-
 # These two functions wrap sourcing and calling respectively.  At present they
 # perform a qa check to make sure eclasses and ebuilds and profiles don't mess
 # with shell opts (shopts).  Ebuilds/eclasses changing shopts should reset them 
@@ -64,22 +66,15 @@ unalias -a
 # Unset some variables that break things.
 unset GZIP BZIP BZIP2 CDPATH GREP_OPTIONS GREP_COLOR GLOBIGNORE
 
-# We need this next line for "die" and "assert". It expands
-# It _must_ preceed all the calls to die and assert.
-shopt -s expand_aliases
-alias die='diefunc "$FUNCNAME" "$LINENO" "$?"'
-alias assert='_pipestatus="${PIPESTATUS[*]}"; [[ "${_pipestatus// /}" -eq 0 ]] || diefunc "$FUNCNAME" "$LINENO" "$_pipestatus"'
-alias save_IFS='[ "${IFS:-unset}" != "unset" ] && old_IFS="${IFS}"'
-alias restore_IFS='if [ "${old_IFS:-unset}" != "unset" ]; then IFS="${old_IFS}"; unset old_IFS; else unset IFS; fi'
-
-OCC="$CC"
-OCXX="$CXX"
-
 export PATH="/usr/local/sbin:/sbin:/usr/sbin:${PORTAGE_BIN_PATH}:/usr/local/bin:/bin:/usr/bin:${ROOTPATH}"
 [ ! -z "$PREROOTPATH" ] && export PATH="${PREROOTPATH%%:}:$PATH"
 
 source "${PORTAGE_BIN_PATH}/isolated-functions.sh"  &>/dev/null
 
+# Set IMAGE for minimal backward compatibility with
+# overlays or user's bashrc, but don't export it.
+[ "${EBUILD_PHASE}" == "preinst" ] && IMAGE=${D}
+
 [[ $PORTAGE_QUIET != "" ]] && export PORTAGE_QUIET
 
 # the sandbox is disabled by default except when overridden in the relevant stages
@@ -114,36 +109,6 @@ lchgrp() {
 	chgrp -h "$@"
 }
 
-# source the existing profile.bashrc's.
-save_IFS
-IFS=$'\n'
-for dir in ${PROFILE_PATHS}; do
-	# Must unset it so that it doesn't mess up assumptions in the RCs.
-	unset IFS
-	if [ -f "${dir}/profile.bashrc" ]; then
-		qa_source "${dir}/profile.bashrc"
-	fi
-done
-restore_IFS
-
-# We assume if people are changing shopts in their bashrc they do so at their
-# own peril.  This is the ONLY non-portage bit of code that can change shopts
-# without a QA violation.
-if [ -f "${PORTAGE_BASHRC}" ]; then
-	# If $- contains x, then tracing has already enabled elsewhere for some
-	# reason.  We preserve it's state so as not to interfere.
-	if [ "$PORTAGE_DEBUG" != "1" ] || [ "${-/x/}" != "$-" ]; then
-		source "${PORTAGE_BASHRC}"
-	else
-		set -x
-		source "${PORTAGE_BASHRC}"
-		set +x
-	fi
-fi
-
-[ ! -z "$OCC" ] && export CC="$OCC"
-[ ! -z "$OCXX" ] && export CXX="$OCXX"
-
 esyslog() {
 	# Custom version of esyslog() to take care of the "Red Star" bug.
 	# MUST follow functions.sh to override the "" parameter problem.
@@ -173,8 +138,15 @@ useq() {
 	fi
 
 	# Make sure we have this USE flag in IUSE
-	if ! hasq "${u}" ${IUSE} ${E_IUSE} && ! hasq "${u}" ${PORTAGE_ARCHLIST} selinux; then
-		eqawarn "QA Notice: USE Flag '${u}' not in IUSE for ${CATEGORY}/${PF}"
+	if [[ -n ${PORTAGE_IUSE} ]] && \
+		[[ -n ${EBUILD_PHASE} ]] && \
+		! hasq ${EBUILD_PHASE} config depend info prerm postrm postinst && \
+		[[ ${EMERGE_FROM} != binary ]] ; then
+		# TODO: Implement PORTAGE_IUSE for binary packages. Currently,
+		# it is only valid for build time phases.
+		echo "${u}" | egrep -q "${PORTAGE_IUSE}" || \
+			eqawarn "QA Notice: USE Flag '${u}' not" \
+				"in IUSE for ${CATEGORY}/${PF}"
 	fi
 
 	if hasq ${u} ${USE} ; then
@@ -190,6 +162,7 @@ has_version() {
 	fi
 	# return shell-true/shell-false if exists.
 	# Takes single depend-type atoms.
+	PYTHONPATH="${PORTAGE_PYM_PATH}:${PYTHONPATH}" \
 	"${PORTAGE_BIN_PATH}"/portageq has_version "${ROOT}" "$1"
 	local retval=$?
 	case "${retval}" in
@@ -209,6 +182,7 @@ portageq() {
 	if [ "${EBUILD_PHASE}" == "depend" ]; then
 		die "portageq calls are not allowed in the global scope"
 	fi
+	PYTHONPATH="${PORTAGE_PYM_PATH}:${PYTHONPATH}" \
 	"${PORTAGE_BIN_PATH}/portageq" "$@"
 }
 
@@ -224,6 +198,7 @@ best_version() {
 	fi
 	# returns the best/most-current match.
 	# Takes single depend-type atoms.
+	PYTHONPATH="${PORTAGE_PYM_PATH}:${PYTHONPATH}" \
 	"${PORTAGE_BIN_PATH}/portageq" 'best_version' "${ROOT}" "$1"
 }
 
@@ -281,108 +256,6 @@ register_die_hook() {
 	export EBUILD_DEATH_HOOKS="${EBUILD_DEATH_HOOKS} $*"
 }
 
-diefunc() {
-	local funcname="$1" lineno="$2" exitcode="$3"
-	shift 3
-
-	local n filespacing=0 linespacing=0
-	# setup spacing to make output easier to read
-	for ((n = ${#FUNCNAME[@]} - 1; n >= 0; --n)); do
-		sourcefile=${BASH_SOURCE[${n}]} sourcefile=${sourcefile##*/}
-		lineno=${BASH_LINENO[${n}]}
-		((filespacing < ${#sourcefile})) && filespacing=${#sourcefile}
-		((linespacing < ${#lineno}))     && linespacing=${#lineno}
-	done
-
-	eerror
-	eerror "ERROR: $CATEGORY/$PF failed."
-	dump_trace 2 ${filespacing} ${linespacing}
-	eerror "  $(printf "%${filespacing}s" "${BASH_SOURCE[1]##*/}"), line $(printf "%${linespacing}s" "${BASH_LINENO[0]}"):  Called die"
-	eerror "The specific snippet of code:"
-	# This scans the file that called die and prints out the logic that
-	# ended in the call to die.  This really only handles lines that end
-	# with '|| die' and any preceding lines with line continuations (\).
-	# This tends to be the most common usage though, so let's do it.
-	# Due to the usage of appending to the hold space (even when empty),
-	# we always end up with the first line being a blank (thus the 2nd sed).
-	sed -n \
-		-e "# When we get to the line that failed, append it to the
-		    # hold space, move the hold space to the pattern space,
-		    # then print out the pattern space and quit immediately
-		    ${BASH_LINENO[0]}{H;g;p;q}" \
-		-e '# If this line ends with a line continuation, append it
-		    # to the hold space
-		    /\\$/H' \
-		-e '# If this line does not end with a line continuation,
-		    # erase the line and set the hold buffer to it (thus
-		    # erasing the hold buffer in the process)
-		    /[^\]$/{s:^.*$::;h}' \
-		${BASH_SOURCE[1]} \
-		| sed -e '1d' -e 's:^:RETAIN-LEADING-SPACE:' \
-		| while read -r n ; do eerror "  ${n#RETAIN-LEADING-SPACE}" ; done
-	eerror " The die message:"
-	eerror "  ${*:-(no error message)}"
-	eerror
-	eerror "If you need support, post the topmost build error, and the call stack if relevant."
-	[[ -n ${PORTAGE_LOG_FILE} ]] \
-		&& eerror "A complete build log is located at '${PORTAGE_LOG_FILE}'."
-	if [[ -n ${EBUILD_OVERLAY_ECLASSES} ]] ; then
-		eerror "This ebuild used the following eclasses from overlays:"
-		local x
-		for x in ${EBUILD_OVERLAY_ECLASSES} ; do
-			eerror "  ${x}"
-		done
-	fi
-	if [[ "${EBUILD#${PORTDIR}/}" == "${EBUILD}" ]] ; then
-		local overlay=${EBUILD%/*}
-		overlay=${overlay%/*}
-		overlay=${overlay%/*}
-		eerror "This ebuild is from an overlay: '${overlay}/'"
-	fi
-	eerror
-
-	if [[ "${EBUILD_PHASE/depend}" == "${EBUILD_PHASE}" ]] ; then
-		local x
-		for x in $EBUILD_DEATH_HOOKS; do
-			${x} "$@" >&2 1>&2
-		done
-	fi
-
-	# subshell die support
-	kill -s SIGTERM ${EBUILD_MASTER_PID}
-	exit 1
-}
-
-shopt -s extdebug &> /dev/null
-
-# dump_trace([number of funcs on stack to skip],
-#            [whitespacing for filenames],
-#            [whitespacing for line numbers])
-dump_trace() {
-	local funcname="" sourcefile="" lineno="" n e s="yes"
-	declare -i strip=${1:-1}
-	local filespacing=$2 linespacing=$3
-
-	eerror "Call stack:"
-	for (( n = ${#FUNCNAME[@]} - 1, p = ${#BASH_ARGV[@]} ; n > ${strip} ; n-- )) ; do
-		funcname=${FUNCNAME[${n} - 1]}
-		sourcefile=$(basename ${BASH_SOURCE[${n}]})
-		lineno=${BASH_LINENO[${n} - 1]}
-		# Display function arguments
-		args=
-		if [[ -n "${BASH_ARGV[@]}" ]]; then
-			for (( j = 1 ; j <= ${BASH_ARGC[${n} - 1]} ; ++j )); do
-				newarg=${BASH_ARGV[$(( p - j - 1 ))]}
-				args="${args:+${args} }'${newarg}'"
-			done
-			(( p -= ${BASH_ARGC[${n} - 1]} ))
-		fi
-		eerror "  $(printf "%${filespacing}s" "${sourcefile}"), line $(printf "%${linespacing}s" "${lineno}"):  Called ${funcname}${args:+ ${args}}"
-	done
-}
-
-
-
 #if no perms are specified, dirs/files will have decent defaults
 #(not secretive, but not stupid)
 umask 022
@@ -428,6 +301,7 @@ keepdir() {
 }
 
 unpack() {
+	local srcdir
 	local x
 	local y
 	local myfail
@@ -439,15 +313,16 @@ unpack() {
 		y=${x%.*}
 		y=${y##*.}
 
-		myfail="${x} does not exist"
-		if [ "${x:0:2}" = "./" ] ; then
+		if [[ ${x} == "./"* ]] ; then
 			srcdir=""
+		elif [[ ${x} == ${DISTDIR%/}/* ]] ; then
+			die "Arguments to unpack() cannot begin with \${DISTDIR}."
+		elif [[ ${x} == "/"* ]] ; then
+			die "Arguments to unpack() cannot be absolute"
 		else
 			srcdir="${DISTDIR}/"
 		fi
-		[[ ${x} == ${DISTDIR}* ]] && \
-			die "Arguments to unpack() should not begin with \${DISTDIR}."
-		[ ! -s "${srcdir}${x}" ] && die "$myfail"
+		[[ ! -s ${srcdir}${x} ]] && die "${x} does not exist"
 
 		myfail="failure unpacking ${x}"
 		case "${x##*.}" in
@@ -534,9 +409,11 @@ econf() {
 	fi
 	if [ -x "${ECONF_SOURCE}/configure" ]; then
 		if [ -e /usr/share/gnuconfig/ ]; then
-			for x in $(find "${WORKDIR}" -type f '(' -name config.guess -o -name config.sub ')') ; do
+			find "${WORKDIR}" -type f '(' \
+			-name config.guess -o -name config.sub ')' -print0 | \
+			while read -d $'\0' x ; do
 				vecho " * econf: updating ${x/${WORKDIR}\/} with /usr/share/gnuconfig/${x##*/}"
-				cp -f /usr/share/gnuconfig/${x##*/} ${x}
+				cp -f /usr/share/gnuconfig/"${x##*/}" "${x}"
 			done
 		fi
 
@@ -626,7 +503,7 @@ econf() {
 
 			if [ -s config.log ]; then
 				echo
-				echo "!!! Please attach the following file when filing a report to bugs.gentoo.org:"
+				echo "!!! Please attach the following file when seeking support:"
 				echo "!!! ${PWD}/config.log"
 			fi
 			die "econf failed"
@@ -677,10 +554,6 @@ einstall() {
 	fi
 }
 
-pkg_setup() {
-	return
-}
-
 pkg_nofetch() {
 	[ -z "${SRC_URI}" ] && return
 
@@ -724,30 +597,6 @@ src_test() {
 	fi
 }
 
-src_install() {
-	return
-}
-
-pkg_preinst() {
-	return
-}
-
-pkg_postinst() {
-	return
-}
-
-pkg_prerm() {
-	return
-}
-
-pkg_postrm() {
-	return
-}
-
-pkg_config() {
-	eerror "This ebuild does not have a config function."
-}
-
 # Used to generate the /lib/cpp and /usr/bin/cc wrappers
 gen_wrapper() {
 	cat > "$1" <<-EOF
@@ -757,10 +606,19 @@ gen_wrapper() {
 	chmod 0755 "$1"
 }
 
+ebuild_phase() {
+	[ "$(type -t ${1})" == "function" ] && qa_call ${1}
+}
+
+ebuild_phase_with_hooks() {
+	local x phase_name=${1}
+	for x in {pre_,,post_}${phase_name} ; do
+		ebuild_phase ${x}
+	done
+}
+
 dyn_setup() {
-	[ "$(type -t pre_pkg_setup)" == "function" ] && qa_call pre_pkg_setup
-	qa_call pkg_setup
-	[ "$(type -t post_pkg_setup)" == "function" ] && qa_call post_pkg_setup
+	ebuild_phase_with_hooks pkg_setup
 }
 
 dyn_unpack() {
@@ -808,7 +666,7 @@ dyn_unpack() {
 	fi
 	cd "${WORKDIR}" || die "Directory change failed: \`cd '${WORKDIR}'\`"
 	vecho ">>> Unpacking source..."
-	qa_call src_unpack
+	ebuild_phase src_unpack
 	touch "${PORTAGE_BUILDDIR}/.unpacked" || die "IO Failure -- Failed 'touch .unpacked' in ${PORTAGE_BUILDDIR}"
 	vecho ">>> Source unpacked."
 	cd "${PORTAGE_BUILDDIR}"
@@ -820,8 +678,9 @@ dyn_clean() {
 	if [ -z "${PORTAGE_BUILDDIR}" ]; then
 		echo "Aborting clean phase because PORTAGE_BUILDDIR is unset!"
 		return 1
+	elif [ ! -d "${PORTAGE_BUILDDIR}" ] ; then
+		return 0
 	fi
-
 	if type -P chflags > /dev/null ; then
 		chflags -R noschg,nouchg,nosappnd,nouappnd "${PORTAGE_BUILDDIR}"
 		chflags -R nosunlnk,nouunlnk "${PORTAGE_BUILDDIR}" 2>/dev/null
@@ -836,6 +695,7 @@ dyn_clean() {
 	fi
 
 	if ! hasq keepwork $FEATURES; then
+		rm -rf "${PORTAGE_BUILDDIR}/.exit_status"
 		rm -rf "${PORTAGE_BUILDDIR}/.logid"
 		rm -rf "${PORTAGE_BUILDDIR}/.unpacked"
 		rm -rf "${PORTAGE_BUILDDIR}/.compiled"
@@ -1019,10 +879,10 @@ dyn_compile() {
 	fi
 	if [ -d "${S}" ]; then
 		srcdir=${S}
-		cd "${S}"
 	else
-		cd "${WORKDIR}"
+		srcdir=${WORKDIR}
 	fi
+	cd "${srcdir}"
 	#our custom version of libtool uses $S and $D to fix
 	#invalid paths in .la files
 	export S D
@@ -1030,35 +890,11 @@ dyn_compile() {
 	#our libtool to create problematic .la files
 	export PWORKDIR="$WORKDIR"
 	vecho ">>> Compiling source in ${srcdir} ..."
-	qa_call src_compile
+	ebuild_phase src_compile
 	vecho ">>> Source compiled."
 	#|| abort_compile "fail"
 	cd "${PORTAGE_BUILDDIR}"
 	touch .compiled
-	cd build-info
-
-	set -f
-	for f in ASFLAGS CATEGORY CBUILD CC CFLAGS CHOST CTARGET CXX \
-		CXXFLAGS DEPEND EXTRA_ECONF EXTRA_EINSTALL EXTRA_MAKE \
-		FEATURES INHERITED IUSE LDFLAGS LIBCFLAGS LIBCXXFLAGS \
-		LICENSE PDEPEND PF PKGUSE PROVIDE RDEPEND RESTRICT SLOT \
-		KEYWORDS HOMEPAGE SRC_URI DESCRIPTION; do
-		[ -n "${!f}" ] && echo $(echo "${!f}" | tr '\n,\r,\t' ' , , ' | sed s/'  \+'/' '/g) > ${f}
-	done
-	echo "${USE}"		> USE
-	echo "${EAPI:-0}"	> EAPI
-	set +f
-	set                     >  environment
-	export | sed 's:^declare -rx:declare -x:' >> environment
-	bzip2 -f9 environment
-
-	cp "${EBUILD}" "${PF}.ebuild"
-	[ -n "${PORTAGE_REPO_NAME}" ]  && echo "${PORTAGE_REPO_NAME}" > repository
-	if hasq nostrip ${FEATURES} ${RESTRICT} || hasq strip ${RESTRICT}
-	then
-		touch DEBUGBUILD
-	fi
-
 	[ "$(type -t post_src_compile)" == "function" ] && qa_call post_src_compile
 
 	trap SIGINT SIGQUIT
@@ -1085,15 +921,12 @@ dyn_test() {
 	fi
 	if ! hasq test $FEATURES && [ "${EBUILD_FORCE_TEST}" != "1" ]; then
 		vecho ">>> Test phase [not enabled]: ${CATEGORY}/${PF}"
-	elif ! hasq test ${USE} && [ "${EBUILD_FORCE_TEST}" != "1" ]; then
-		ewarn "Skipping make test/check since USE=test is masked."
-		vecho ">>> Test phase [explicitly disabled]: ${CATEGORY}/${PF}"
 	elif hasq test $RESTRICT; then
 		ewarn "Skipping make test/check due to ebuild restriction."
 		vecho ">>> Test phase [explicitly disabled]: ${CATEGORY}/${PF}"
 	else
 		addpredict /
-		qa_call src_test
+		ebuild_phase src_test
 		SANDBOX_PREDICT="${SANDBOX_PREDICT%:/}"
 	fi
 
@@ -1129,27 +962,57 @@ dyn_install() {
 	#some packages uses an alternative to $S to build in, cause
 	#our libtool to create problematic .la files
 	export PWORKDIR="$WORKDIR"
-	qa_call src_install
+	ebuild_phase src_install
 	touch "${PORTAGE_BUILDDIR}/.installed"
 	vecho ">>> Completed installing ${PF} into ${D}"
 	vecho
 	cd ${PORTAGE_BUILDDIR}
 	[ "$(type -t post_src_install)" == "function" ] && qa_call post_src_install
+
+	cd "${PORTAGE_BUILDDIR}"/build-info
+	set -f
+	local f
+	for f in ASFLAGS CATEGORY CBUILD CC CFLAGS CHOST CTARGET CXX \
+		CXXFLAGS DEPEND EXTRA_ECONF EXTRA_EINSTALL EXTRA_MAKE \
+		FEATURES INHERITED IUSE LDFLAGS LIBCFLAGS LIBCXXFLAGS \
+		LICENSE PDEPEND PF PKGUSE PROVIDE RDEPEND RESTRICT SLOT \
+		KEYWORDS HOMEPAGE SRC_URI DESCRIPTION; do
+		[ -n "${!f}" ] && echo $(echo "${!f}" | \
+			tr '\n,\r,\t' ' , , ' | sed s/'  \+'/' '/g) > ${f}
+	done
+	echo "${USE}"       > USE
+	echo "${EAPI:-0}"   > EAPI
+	set +f
+
+	# local variables can leak into the saved environment.
+	unset f
+
+	(
+		# To avoid environment.bz2 bloat, cleanse variables that are
+		# are no longer needed after src_install(). Don't cleanse from
+		# the global environment though, in case the user wants to repeat
+		# this phase (like with FEATURES=noauto and the ebuild command).
+		unset S _E_DOCDESTTREE_ _E_EXEDESTTREE_
+		save_ebuild_env | filter_readonly_variables \
+			--filter-sandbox --allow-extra-vars > environment
+	)
+	bzip2 -f9 environment
+
+	cp "${EBUILD}" "${PF}.ebuild"
+	[ -n "${PORTAGE_REPO_NAME}" ]  && echo "${PORTAGE_REPO_NAME}" > repository
+	if hasq nostrip ${FEATURES} ${RESTRICT} || hasq strip ${RESTRICT}
+	then
+		touch DEBUGBUILD
+	fi
 	trap SIGINT SIGQUIT
 }
 
 dyn_preinst() {
-	if [ -z "$IMAGE" ]; then
-		eerror "${FUNCNAME}: IMAGE is unset"
+	if [ -z "${D}" ]; then
+		eerror "${FUNCNAME}: D is unset"
 		return 1
 	fi
-
-	[ "$(type -t pre_pkg_preinst)" == "function" ] && qa_call pre_pkg_preinst
-
-	declare -r D=${IMAGE}
-	pkg_preinst
-
-	[ "$(type -t post_pkg_preinst)" == "function" ] && qa_call post_pkg_preinst
+	ebuild_phase_with_hooks pkg_preinst
 }
 
 dyn_help() {
@@ -1461,15 +1324,189 @@ remove_path_entry() {
 	PATH="${stripped_path}"
 }
 
+source_all_bashrcs() {
+	local OCC="${CC}" OCXX="${CXX}"
+	# source the existing profile.bashrc's.
+	save_IFS
+	IFS=$'\n'
+	local x
+	for x in ${PROFILE_PATHS}; do
+		# Must unset it so that it doesn't mess up assumptions in the RCs.
+		unset IFS
+		[ -f "${x}/profile.bashrc" ] && qa_source "${x}/profile.bashrc"
+	done
+	restore_IFS
+	
+	# We assume if people are changing shopts in their bashrc they do so at their
+	# own peril.  This is the ONLY non-portage bit of code that can change shopts
+	# without a QA violation.
+	if [ -f "${PORTAGE_BASHRC}" ]; then
+		# If $- contains x, then tracing has already enabled elsewhere for some
+		# reason.  We preserve it's state so as not to interfere.
+		if [ "$PORTAGE_DEBUG" != "1" ] || [ "${-/x/}" != "$-" ]; then
+			source "${PORTAGE_BASHRC}"
+		else
+			set -x
+			source "${PORTAGE_BASHRC}"
+			set +x
+		fi
+	fi
+	[ ! -z "${OCC}" ] && export CC="${OCC}"
+	[ ! -z "${OCXX}" ] && export CXX="${OCXX}"
+}
+
+# Hardcoded bash lists are needed for backward compatibility with
+# <portage-2.1.4 since they assume that a newly installed version
+# of ebuild.sh will work for pkg_postinst, pkg_prerm, and pkg_postrm
+# when portage is upgrading itself.
+
+READONLY_EBUILD_METADATA="DEPEND DESCRIPTION
+	EAPI HOMEPAGE INHERITED IUSE KEYWORDS LICENSE
+	PDEPEND PROVIDE RDEPEND RESTRICT SLOT SRC_URI"
+
+READONLY_PORTAGE_VARS="D EBUILD EBUILD_PHASE \
+	EBUILD_SH_ARGS EMERGE_FROM FILESDIR PORTAGE_BINPKG_FILE \
+	PORTAGE_BIN_PATH PORTAGE_IUSE \
+	PORTAGE_PYM_PATH PORTAGE_MUTABLE_FILTERED_VARS \
+	PORTAGE_SAVED_READONLY_VARS PORTAGE_TMPDIR T WORKDIR"
+
+PORTAGE_SAVED_READONLY_VARS="A CATEGORY P PF PN PR PV PVR"
+
+# Variables that portage sets but doesn't mark readonly.
+# In order to prevent changed values from causing unexpected
+# interference, they are filtered out of the environment when
+# it is saved or loaded (any mutations do not persist).
+PORTAGE_MUTABLE_FILTERED_VARS="AA HOSTNAME"
+
+# @FUNCTION: filter_readonly_variables
+# @DESCRIPTION: [--filter-sandbox] [--allow-extra-vars]
+# Read an environment from stdin and echo to stdout while filtering readonly
+# variables.
+#
+# --filter-sandbox causes all SANDBOX_* variables to be filtered, which
+# is only desired in certain cases, such as during preprocessing or when
+# saving environment.bz2 for a binary or installed package.
+#
+# --filter-features causes the special FEATURES variable to be filtered.
+# Generally, we want it to persist between phases since the user might
+# want to modify it via bashrc to enable things like splitdebug and
+# installsources for specific packages. They should be able to modify it
+# in pre_pkg_setup() and have it persist all the way through the install
+# phase. However, if FEATURES exist inside environment.bz2 then they
+# should be overridden by current settings.
+#
+# ---allow-extra-vars causes some extra vars to be allowd through, such
+# as ${PORTAGE_SAVED_READONLY_VARS} and ${PORTAGE_MUTABLE_FILTERED_VARS}.
+#
+# In bash-3.2_p20+ an attempt to assign BASH_*, FUNCNAME, GROUPS or any
+# readonly variable cause the shell to exit while executing the "source"
+# builtin command. To avoid this problem, this function filters those
+# variables out and discards them. See bug #190128.
+filter_readonly_variables() {
+	local x filtered_vars var_grep
+	local readonly_bash_vars="DIRSTACK EUID FUNCNAME GROUPS
+		PIPESTATUS PPID SHELLOPTS UID"
+	local filtered_sandbox_vars="SANDBOX_ACTIVE SANDBOX_BASHRC
+		SANDBOX_DEBUG_LOG SANDBOX_DISABLED SANDBOX_LIB
+		SANDBOX_LOG SANDBOX_ON"
+	filtered_vars="${readonly_bash_vars} ${READONLY_PORTAGE_VARS}
+		BASH_[_[:alnum:]]* PATH"
+	if hasq --filter-sandbox $* ; then
+		filtered_vars="${filtered_vars} SANDBOX_[_[:alnum:]]*"
+	else
+		filtered_vars="${filtered_vars} ${filtered_sandbox_vars}"
+	fi
+	if hasq --filter-features $* ; then
+		filtered_vars="${filtered_vars} FEATURES"
+	fi
+	if ! hasq --allow-extra-vars $* ; then
+		filtered_vars="
+			${filtered_vars}
+			${PORTAGE_SAVED_READONLY_VARS}
+			${PORTAGE_MUTABLE_FILTERED_VARS}
+		"
+	fi
+	set -f
+	for x in ${filtered_vars} ; do
+		var_grep="${var_grep}|${x}"
+	done
+	set +f
+	var_grep=${var_grep:1} # strip the first |
+	var_grep="(^|^declare[[:space:]]+-[^[:space:]]+[[:space:]]+|^export[[:space:]]+)(${var_grep})=.*"
+	# The sed is to remove the readonly attribute from variables such as those
+	# listed in READONLY_EBUILD_METADATA, since having any readonly attributes
+	# persisting in the saved environment can be inconvenient when it
+	# eventually needs to be reloaded.
+	"${PORTAGE_BIN_PATH}"/filter-bash-environment.py "${var_grep}" | sed -r \
+		-e 's:^declare[[:space:]]+-r[[:space:]]+:declare :' \
+		-e 's:^declare[[:space:]]+-([[:alnum:]]*)r([[:alnum:]]*)[[:space:]]+:declare -\1\2 :'
+}
+
+# @FUNCTION: preprocess_ebuild_env
+# @DESCRIPTION:
+# Filter any readonly variables from ${T}/environment, source it, and then
+# save it via save_ebuild_env(). This process should be sufficient to prevent
+# any stale variables or functions from an arbitrary environment from
+# interfering with the current environment. This is useful when an existing
+# environment needs to be loaded from a binary or installed package.
+preprocess_ebuild_env() {
+	local filter_opts=""
+	if [ -f "${T}/environment.raw" ] ; then
+		# This is a signal from the python side, indicating that the
+		# environment may contain stale SANDBOX_{DENY,PREDICT,READ,WRITE}
+		# and FEATURES variables that should be filtered out. Between
+		# phases, these variables are normally preserved.
+		filter_opts="--filter-sandbox --filter-features ${filter_opts}"
+	fi
+	filter_readonly_variables ${filter_opts} < "${T}"/environment \
+		> "${T}"/environment.filtered || return $?
+	unset filter_opts
+	mv "${T}"/environment.filtered "${T}"/environment || return $?
+	rm -f "${T}/environment.success" || return $?
+	# WARNING: Code inside this subshell should avoid making assumptions
+	# about variables or functions after source "${T}"/environment has been
+	# called. Any variables that need to be relied upon should already be
+	# filtered out above.
+	(
+		export SANDBOX_ON=1
+		source "${T}/environment" || exit $?
+		# We have to temporarily disable sandbox since the
+		# SANDBOX_{DENY,READ,PREDICT,WRITE} values we've just loaded
+		# may be unusable (triggering in spurious sandbox violations)
+		# until we've merged them with our current values.
+		export SANDBOX_ON=0
+
+		# It's remotely possible that save_ebuild_env() has been overridden
+		# by the above source command. To protect ourselves, we override it
+		# here with our own version. ${PORTAGE_BIN_PATH} is safe to use here
+		# because it's already filtered above.
+		source "${PORTAGE_BIN_PATH}/isolated-functions.sh" || exit $?
+
+		# Rely on save_ebuild_env() to filter out any remaining variables
+		# and functions that could interfere with the current environment.
+		save_ebuild_env || exit $?
+		touch "${T}/environment.success" || exit $?
+	) > "${T}/environment.filtered"
+	local retval
+	if [ -e "${T}/environment.success" ] ; then
+		filter_readonly_variables < \
+			"${T}/environment.filtered" > "${T}/environment"
+		retval=$?
+	else
+		retval=1
+	fi
+	rm -f "${T}"/environment.{filtered,raw,success}
+	return ${retval}
+}
+
 # === === === === === === === === === === === === === === === === === ===
 # === === === === === functions end, main part begins === === === === ===
 # === === === === === functions end, main part begins === === === === ===
 # === === === === === functions end, main part begins === === === === ===
 # === === === === === === === === === === === === === === === === === ===
 
-if [[ ${EBUILD_SH_ARGS} != "depend" ]] && [[ ${EBUILD_SH_ARGS}  != "clean" ]] && [[ ${EBUILD_SH_ARGS} != "setup" ]]; then
-	cd ${PORTAGE_TMPDIR} &> /dev/null
-	cd ${BUILD_PREFIX} &> /dev/null
+if [ -n "${EBUILD_SH_ARGS}" ] && \
+	! hasq ${EBUILD_SH_ARGS} clean depend help info nofetch ; then
 
 	if [ "$(id -nu)" == "portage" ] ; then
 		export USER=portage
@@ -1539,13 +1576,6 @@ export S=${WORKDIR}/${P}
 
 unset E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
 
-for x in T P PN PV PVR PR CATEGORY A EBUILD EMERGE_FROM O PPID FILESDIR PORTAGE_TMPDIR; do
-	[[ ${!x-UNSET_VAR} != UNSET_VAR ]] && declare -r ${x}
-done
-# Need to be able to change D in dyn_preinst due to the IMAGE stuff
-[[ ${EBUILD_SH_ARGS} != "preinst" ]] && declare -r D
-unset x
-
 # Turn of extended glob matching so that g++ doesn't get incorrectly matched.
 shopt -u extglob
 
@@ -1572,27 +1602,112 @@ if hasq "depend" "${EBUILD_SH_ARGS}"; then
 	unset BIN_PATH BIN BODY FUNC_SRC
 fi
 
-# reset the EBUILD_DEATH_HOOKS so they don't multiple due to stable's re-sourcing of env.
-# this can be left out of ebd variants, since they're unaffected.
-unset EBUILD_DEATH_HOOKS
+if ! hasq ${EBUILD_PHASE} clean depend && \
+	[ -f "${T}"/environment ] ; then
+	# The environment may have been extracted from environment.bz2 or
+	# may have come from another version of ebuild.sh or something.
+	# In any case, preprocess it to prevent any potential interference.
+	preprocess_ebuild_env || \
+		die "error processing environment"
+	# Colon separated SANDBOX_* variables need to be cumulative.
+	for x in SANDBOX_DENY SANDBOX_READ SANDBOX_PREDICT SANDBOX_WRITE ; do
+		eval PORTAGE_${x}=\${!x}
+	done
+	PORTAGE_SANDBOX_ON=${SANDBOX_ON}
+	export SANDBOX_ON=1
+	source "${T}"/environment || \
+		die "error sourcing environment"
+	# We have to temporarily disable sandbox since the
+	# SANDBOX_{DENY,READ,PREDICT,WRITE} values we've just loaded
+	# may be unusable (triggering in spurious sandbox violations)
+	# until we've merged them with our current values.
+	export SANDBOX_ON=0
+	for x in SANDBOX_DENY SANDBOX_PREDICT SANDBOX_READ SANDBOX_WRITE ; do
+		eval y=\${PORTAGE_${x}}
+		if [ "${y}" != "${!x}" ] ; then
+			eval export ${x}=\"$(echo -n "${y}:${!x}" | tr ":" "\0" | \
+				sort -z -u | tr "\0" ":")\"
+		fi
+		unset PORTAGE_${x}
+	done
+	unset x y
+	export SANDBOX_ON=${PORTAGE_SANDBOX_ON}
+	unset PORTAGE_SANDBOX_ON
 
-# *DEPEND and IUSE will be set during the sourcing of the ebuild.  In order to
-# ensure correct interaction between ebuilds and eclasses, they need to be
-# unset before this process of interaction begins.
-unset DEPEND RDEPEND PDEPEND IUSE
+	# After loading the environment, make sure the color variables
+	# are in sync with the current NOCOLOR setting.
+	case "${NOCOLOR:-false}" in
+		yes|true)
+			unset_colors
+			;;
+		no|false)
+			set_colors
+			;;
+	esac
 
-source ${EBUILD} || die "error sourcing ebuild"
-if ! hasq depend $EBUILD_PHASE; then
-	RESTRICT="${PORTAGE_RESTRICT}"
-	unset PORTAGE_RESTRICT
+	source_all_bashrcs
 fi
 
-# Expand KEYWORDS
-# We need to turn off pathname expansion for -* in KEYWORDS and
-# we need to escape ~ to avoid tilde expansion
-set -f
-KEYWORDS=$(eval echo ${KEYWORDS//~/\\~})
-set +f
+if ! hasq ${EBUILD_PHASE} clean && \
+	(
+		hasq ${EBUILD_PHASE} depend || \
+		[ ! -f "${T}"/environment ] || \
+		hasq noauto ${FEATURES}
+	) ; then
+	# The bashrcs get an opportunity here to set aliases that will be expanded
+	# during sourcing of ebuilds and eclasses.
+	source_all_bashrcs
+
+	# *DEPEND and IUSE will be set during the sourcing of the ebuild.
+	# In order to ensure correct interaction between ebuilds and
+	# eclasses, they need to be unset before this process of
+	# interaction begins.
+	unset DEPEND RDEPEND PDEPEND IUSE
+	source "${EBUILD}" || die "error sourcing ebuild"
+
+	if [ "${EBUILD_PHASE}" != "depend" ] ; then
+		RESTRICT=${PORTAGE_RESTRICT}
+	fi
+
+	# This next line is not the same as export RDEPEND=${RDEPEND:-${DEPEND}}
+	# That will test for unset *or* NULL (""). We want just to set for unset...
+	# turn off glob expansion from here on in to prevent *'s and ? in the
+	# DEPEND syntax from getting expanded :)
+	set -f
+	if [ "${RDEPEND-unset}" == "unset" ] ; then
+		export RDEPEND=${DEPEND}
+		debug-print "RDEPEND: not set... Setting to: ${DEPEND}"
+	fi
+
+	# add in dependency info from eclasses
+	IUSE="${IUSE} ${E_IUSE}"
+	DEPEND="${DEPEND} ${E_DEPEND}"
+	RDEPEND="${RDEPEND} ${E_RDEPEND}"
+	PDEPEND="${PDEPEND} ${E_PDEPEND}"
+
+	unset ECLASS E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
+
+	if [ "${EBUILD_PHASE}" != "depend" ] ; then
+		# Make IUSE defaults backward compatible with all the old shell code.
+		iuse_temp=""
+		for x in ${IUSE} ; do
+			if [[ ${x} == +* ]] || [[ ${x} == -* ]] ; then
+				iuse_temp="${iuse_temp} ${x:1}"
+			else
+				iuse_temp="${iuse_temp} ${x}"
+			fi
+		done
+		export IUSE=${iuse_temp}
+		unset x iuse_temp
+	fi
+	set +f
+fi
+
+# unset USE_EXPAND variables that contain only the special "*" token
+for x in ${USE_EXPAND} ; do
+	[ "${!x}" == "*" ] && unset ${x}
+done
+unset x
 
 if hasq nostrip ${FEATURES} ${RESTRICT} || hasq strip ${RESTRICT}
 then
@@ -1614,74 +1729,29 @@ fi
 export TMP="${T}"
 export TMPDIR="${T}"
 
-# Note: this next line is not the same as export RDEPEND=${RDEPEND:-${DEPEND}}
-# That will test for unset *or* NULL ("").  We want just to set for unset...
-
-#turn off glob expansion from here on in to prevent *'s and ? in the DEPEND
-#syntax from getting expanded :)
-#check eclass rdepends also.
-set -f
-if [ "${RDEPEND-unset}" == "unset" ] ; then
-	export RDEPEND=${DEPEND}
-	debug-print "RDEPEND: not set... Setting to: ${DEPEND}"
+# Note: readonly variables interfere with preprocess_ebuild_env(), so
+# declare them only after it has already run.
+if [ "${EBUILD_PHASE}" != "depend" ] ; then
+	declare -r ${READONLY_EBUILD_METADATA} ${READONLY_PORTAGE_VARS}
 fi
 
-#add in dependency info from eclasses
-IUSE="$IUSE $E_IUSE"
-DEPEND="$DEPEND $E_DEPEND"
-RDEPEND="$RDEPEND $E_RDEPEND"
-PDEPEND="$PDEPEND $E_PDEPEND"
-
-unset E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
-
-if [ "${EBUILD_PHASE}" != "depend" ]; then
-	# Make IUSE defaults backward compatible with all the old shell code.
-	iuse_temp=""
-	for x in ${IUSE} ; do
-		if [[ ${x} == +* ]] || [[ ${x} == -* ]] ; then
-			iuse_temp="${iuse_temp} ${x:1}"
-		else
-			iuse_temp="${iuse_temp} ${x}"
-		fi
-	done
-	export IUSE=${iuse_temp}
-	unset iuse_temp
-	# unset USE_EXPAND variables that contain only the special "*" token
-	for x in ${USE_EXPAND} ; do
-		[ "${!x}" == "*" ] && unset ${x}
-	done
-	unset x
-	# Lock the dbkey variables after the global phase
-	declare -r DEPEND RDEPEND SLOT SRC_URI RESTRICT HOMEPAGE LICENSE DESCRIPTION
-	declare -r KEYWORDS INHERITED IUSE PDEPEND PROVIDE
-fi
-
-set +f
-
-for myarg in ${EBUILD_SH_ARGS} ; do
-	case $myarg in
+if [ -n "${EBUILD_SH_ARGS}" ] ; then
+	case ${EBUILD_SH_ARGS} in
 	nofetch)
-		qa_call pkg_nofetch
+		ebuild_phase_with_hooks pkg_nofetch
 		exit 1
 		;;
 	prerm|postrm|postinst|config|info)
-		if [ "${myarg}" == "info" ] && \
-			[ "$(type -t pkg_${myarg})" != "function" ]; then
-			ewarn  "pkg_${myarg}() is not defined: '${EBUILD##*/}'"
-			continue
+		if hasq ${EBUILD_SH_ARGS} config info && \
+			[ "$(type -t pkg_${EBUILD_SH_ARGS})" != "function" ]; then
+			ewarn  "pkg_${EBUILD_SH_ARGS}() is not defined: '${EBUILD##*/}'"
 		fi
 		export SANDBOX_ON="0"
-		if [ "$PORTAGE_DEBUG" != "1" ]; then
-			[ "$(type -t pre_pkg_${myarg})" == "function" ] && qa_call pre_pkg_${myarg}
-			qa_call pkg_${myarg}
-			[ "$(type -t post_pkg_${myarg})" == "function" ] && qa_call post_pkg_${myarg}
-			#Allow non-zero return codes since they can be caused by &&
+		if [ "${PORTAGE_DEBUG}" != "1" ] || [ "${-/x/}" != "$-" ]; then
+			ebuild_phase_with_hooks pkg_${EBUILD_SH_ARGS}
 		else
 			set -x
-			[ "$(type -t pre_pkg_${myarg})" == "function" ] && qa_call pre_pkg_${myarg}
-			qa_call pkg_${myarg}
-			[ "$(type -t post_pkg_${myarg})" == "function" ] && qa_call post_pkg_${myarg}
-			#Allow non-zero return codes since they can be caused by &&
+			ebuild_phase_with_hooks pkg_${EBUILD_SH_ARGS}
 			set +x
 		fi
 		;;
@@ -1691,13 +1761,11 @@ for myarg in ${EBUILD_SH_ARGS} ; do
 		else
 			export SANDBOX_ON="0"
 		fi
-		if [ "$PORTAGE_DEBUG" != "1" ]; then
-			dyn_${myarg}
-			#Allow non-zero return codes since they can be caused by &&
+		if [ "${PORTAGE_DEBUG}" != "1" ] || [ "${-/x/}" != "$-" ]; then
+			dyn_${EBUILD_SH_ARGS}
 		else
 			set -x
-			dyn_${myarg}
-			#Allow non-zero return codes since they can be caused by &&
+			dyn_${EBUILD_SH_ARGS}
 			set +x
 		fi
 		export SANDBOX_ON="0"
@@ -1707,11 +1775,11 @@ for myarg in ${EBUILD_SH_ARGS} ; do
 		#for example, awking and piping a file in /tmp requires a temp file to be created
 		#in /etc.  If pkg_setup is in the sandbox, both our lilo and apache ebuilds break.
 		export SANDBOX_ON="0"
-		if [ "$PORTAGE_DEBUG" != "1" ]; then
-			dyn_${myarg}
+		if [ "${PORTAGE_DEBUG}" != "1" ] || [ "${-/x/}" != "$-" ]; then
+			dyn_${EBUILD_SH_ARGS}
 		else
 			set -x
-			dyn_${myarg}
+			dyn_${EBUILD_SH_ARGS}
 			set +x
 		fi
 		;;
@@ -1747,35 +1815,25 @@ for myarg in ${EBUILD_SH_ARGS} ; do
 			9>&-
 		fi
 		set +f
-		#make sure it is writable by our group:
-		exit 0
 		;;
 	*)
 		export SANDBOX_ON="1"
-		echo "Please specify a valid command."
+		echo "Unrecognized EBUILD_SH_ARGS: '${EBUILD_SH_ARGS}'"
 		echo
 		dyn_help
 		exit 1
 		;;
 	esac
-
-	#if [ $? -ne 0 ]; then
-	#	exit 1
-	#fi
-done
+	[ -n "${EBUILD_EXIT_STATUS_FILE}" ] && \
+		touch "${EBUILD_EXIT_STATUS_FILE}" &>/dev/null
+fi
 
 # Save the env only for relevant phases.
-if [ -n "${myarg}" ] && \
-	[ "${myarg}" != "clean" ] && \
-	[ "${myarg}" != "help" ] ; then
-	# Do not save myarg in the env, or else the above [ -n "$myarg" ] test will
-	# give a false positive when ebuild.sh is sourced.
-	unset myarg
+if [ -n "${EBUILD_SH_ARGS}" ] && \
+	! hasq ${EBUILD_SH_ARGS} clean depend help info nofetch ; then
 	# Save current environment and touch a success file. (echo for success)
 	umask 002
-	set | egrep -v -e "^SANDBOX_" -e "^LD_PRELOAD=" -e "^FAKEROOTKEY=" > "${T}/environment" 2>/dev/null
-	export | egrep -v -e "^declare -x SANDBOX_" -e "^declare -x LD_PRELOAD=" -e "^declare -x FAKEROOTKEY=" | \
-		sed 's:^declare -rx:declare -x:' >> "${T}/environment" 2>/dev/null
+	save_ebuild_env | filter_readonly_variables > "${T}/environment"
 	chown portage:portage "${T}/environment" &>/dev/null
 	chmod g+w "${T}/environment" &>/dev/null
 fi
