@@ -314,12 +314,20 @@ install_qa_check() {
 			vecho -ne '\a\n'
 			abort="yes"
 		fi
+		if [[ ${abort} == "yes" ]] ; then
+			echo "Please do not file a Gentoo bug and instead" \
+			"report the above QA issues directly to the upstream" \
+			"developers of this software." | fmt -w 70 | \
+			while read line ; do eqawarn "${line}" ; done
+			eqawarn "Homepage: ${HOMEPAGE}"
+		fi
 		[[ ${abort} == "yes" ]] && hasq stricter ${FEATURES} && die "poor code kills airplanes"
 	fi
 
 	# Compiled python objects do not belong in /usr/share (FHS violation)
 	# and can be a pain when upgrading python
-	f=$([ -d "${D}"/usr/share ] && find "${D}"/usr/share -name '*.py[co]')
+	f=$([ -d "${D}"/usr/share ] && \
+		find "${D}"usr/share -name '*.py[co]' | sed "s:${D}:/:")
 	if [[ -n ${f} ]] ; then
 		vecho -ne '\a\n'
 		eqawarn "QA Notice: Precompiled python object files do not belong in /usr/share"
@@ -395,8 +403,8 @@ postinst_bsdflags() {
 }
 
 preinst_mask() {
-	if [ -z "$IMAGE" ]; then
-		 eerror "${FUNCNAME}: IMAGE is unset"
+	if [ -z "${D}" ]; then
+		 eerror "${FUNCNAME}: D is unset"
 		 return 1
 	fi
 	# remove man pages, info pages, docs if requested
@@ -406,62 +414,77 @@ preinst_mask() {
 		fi
 	done
 
-	install_mask "${IMAGE}" ${INSTALL_MASK}
+	install_mask "${D}" ${INSTALL_MASK}
 
 	# remove share dir if unnessesary
 	if hasq nodoc $FEATURES -o hasq noman $FEATURES -o hasq noinfo $FEATURES; then
-		rmdir "${IMAGE}/usr/share" &> /dev/null
+		rmdir "${D}usr/share" &> /dev/null
 	fi
 }
 
 preinst_sfperms() {
-	if [ -z "$IMAGE" ]; then
-		 eerror "${FUNCNAME}: IMAGE is unset"
+	if [ -z "${D}" ]; then
+		 eerror "${FUNCNAME}: D is unset"
 		 return 1
 	fi
 	# Smart FileSystem Permissions
 	if hasq sfperms $FEATURES; then
-		for i in $(find ${IMAGE}/ -type f -perm -4000); do
-			ebegin ">>> SetUID: [chmod go-r] $i "
-			chmod go-r "$i"
-			eend $?
+		local i
+		find "${D}" -type f -perm -4000 -print0 | \
+		while read -d $'\0' i ; do
+			if [ -n "$(find "$i" -perm -2000)" ] ; then
+				ebegin ">>> SetUID and SetGID: [chmod o-r] /${i#${D}}"
+				chmod o-r "$i"
+				eend $?
+			else
+				ebegin ">>> SetUID: [chmod go-r] /${i#${D}}"
+				chmod go-r "$i"
+				eend $?
+			fi
 		done
-		for i in $(find ${IMAGE}/ -type f -perm -2000); do
-			ebegin ">>> SetGID: [chmod o-r] $i "
-			chmod o-r "$i"
-			eend $?
+		find "${D}" -type f -perm -2000 -print0 | \
+		while read -d $'\0' i ; do
+			if [ -n "$(find "$i" -perm -4000)" ] ; then
+				# This case is already handled
+				# by the SetUID check above.
+				true
+			else
+				ebegin ">>> SetGID: [chmod o-r] /${i#${D}}"
+				chmod o-r "$i"
+				eend $?
+			fi
 		done
 	fi
 }
 
 preinst_suid_scan() {
-	if [ -z "$IMAGE" ]; then
-		 eerror "${FUNCNAME}: IMAGE is unset"
+	if [ -z "${D}" ]; then
+		 eerror "${FUNCNAME}: D is unset"
 		 return 1
 	fi
 	# total suid control.
 	if hasq suidctl $FEATURES; then
-		sfconf=/etc/portage/suidctl.conf
-		vecho ">>> Performing suid scan in ${IMAGE}"
-		for i in $(find ${IMAGE}/ -type f \( -perm -4000 -o -perm -2000 \) ); do
+		sfconf=${PORTAGE_CONFIGROOT}etc/portage/suidctl.conf
+		vecho ">>> Performing suid scan in ${D}"
+		for i in $(find "${D}" -type f \( -perm -4000 -o -perm -2000 \) ); do
 			if [ -s "${sfconf}" ]; then
-				suid="$(grep ^${i/${IMAGE}/}$ ${sfconf})"
-				if [ "${suid}" = "${i/${IMAGE}/}" ]; then
-					vecho "- ${i/${IMAGE}/} is an approved suid file"
+				suid="$(grep "^${i/${D}}$" "${sfconf}")"
+				if [ "${suid}" = "${i/${D}}" ]; then
+					vecho "- ${i/${D}} is an approved suid file"
 				else
-					vecho ">>> Removing sbit on non registered ${i/${IMAGE}/}"
+					vecho ">>> Removing sbit on non registered ${i/${D}}"
 					for x in 5 4 3 2 1 0; do echo -ne "\a"; sleep 0.25 ; done
 					vecho -ne "\a"
 					ls_ret=$(ls -ldh "${i}")
 					chmod ugo-s "${i}"
-					grep ^#${i/${IMAGE}/}$ ${sfconf} > /dev/null || {
+					grep "^#${i/${D}}$" "${sfconf}" > /dev/null || {
 						# sandbox prevents us from writing directly
 						# to files outside of the sandbox, but this
 						# can easly be bypassed using the addwrite() function
 						addwrite "${sfconf}"
 						vecho ">>> Appending commented out entry to ${sfconf} for ${PF}"
-						echo "## ${ls_ret%${IMAGE}*}${ls_ret#*${IMAGE}}" >> ${sfconf}
-						echo "#${i/${IMAGE}/}" >> ${sfconf}
+						echo "## ${ls_ret%${D}*}${ls_ret#*${D}}" >> "${sfconf}"
+						echo "#${i/${D}}" >> "${sfconf}"
 						# no delwrite() eh?
 						# delwrite ${sconf}
 					}
@@ -474,8 +497,8 @@ preinst_suid_scan() {
 }
 
 preinst_selinux_labels() {
-	if [ -z "$IMAGE" ]; then
-		 eerror "${FUNCNAME}: IMAGE is unset"
+	if [ -z "${D}" ]; then
+		 eerror "${FUNCNAME}: D is unset"
 		 return 1
 	fi
 	if hasq selinux ${FEATURES}; then
@@ -490,7 +513,7 @@ preinst_selinux_labels() {
 	
 				addwrite /selinux/context;
 	
-				/usr/sbin/setfiles "${file_contexts_path}" -r "${IMAGE}" "${IMAGE}";
+				/usr/sbin/setfiles "${file_contexts_path}" -r "${D}" "${D}"
 			) || die "Failed to set SELinux security labels."
 		else
 			# nonfatal, since merging can happen outside a SE kernel
@@ -521,6 +544,16 @@ dyn_package() {
 	fi
 	mv -f "${pkg_tmp}" "${pkg_dest}" || die "Failed to move tbz2 to ${pkg_dest}"
 	ln -sf "../All/${PF}.tbz2" "${PKGDIR}/${CATEGORY}/${PF}.tbz2" || die "Failed to create symlink in ${PKGDIR}/${CATEGORY}"
+	local md5_hash=""
+	if type md5sum &>/dev/null ; then
+		md5_hash=$(md5sum "${pkg_dest}")
+		md5_hash=${md5_hash%% *}
+	elif type md5 &>/dev/null ; then
+		md5_hash=$(md5 "${pkg_dest}")
+		md5_hash=${md5_hash##* }
+	fi
+	[ -n "${md5_hash}" ] && \
+		echo ${md5_hash} > "${PORTAGE_BUILDDIR}"/build-info/BINPKGMD5
 	vecho ">>> Done."
 	cd "${PORTAGE_BUILDDIR}"
 	touch .packaged || die "Failed to 'touch .packaged' in ${PORTAGE_BUILDDIR}"
@@ -577,4 +610,7 @@ if [ -n "${MISC_FUNCTIONS_ARGS}" ]; then
 	done
 fi
 
+[ -n "${EBUILD_EXIT_STATUS_FILE}" ] && \
+	touch "${EBUILD_EXIT_STATUS_FILE}" &>/dev/null
+
 :
