@@ -1,7 +1,7 @@
 # portage.py -- core Portage functionality
 # Copyright 1998-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_exec.py 7380 2007-07-24 00:52:59Z zmedico $
+# $Id: portage_exec.py 9019 2007-12-21 02:21:07Z zmedico $
 
 
 import os, atexit, signal, sys
@@ -181,7 +181,11 @@ def spawn(mycommand, env={}, opt_name=None, fd_pipes=None, returnpid=False,
 	# If we haven't been told what file descriptors to use
 	# default to propogating our stdin, stdout and stderr.
 	if fd_pipes is None:
-		fd_pipes = {0:0, 1:1, 2:2}
+		fd_pipes = {
+			0:sys.stdin.fileno(),
+			1:sys.stdout.fileno(),
+			2:sys.stderr.fileno(),
+		}
 
 	# mypids will hold the pids of all processes created.
 	mypids = []
