@@ -1,43 +1,27 @@
 # portage: Lock management code
 # Copyright 2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/portage/pym/portage_locks.py,v 1.18.2.2 2005/01/16 02:35:33 carpaski Exp $
-cvs_id_string="$Id: portage_locks.py,v 1.18.2.2 2005/01/16 02:35:33 carpaski Exp $"[5:-2]
+# $Id: portage_locks.py 6932 2007-06-22 02:34:17Z zmedico $
+
 
-import atexit
 import errno
 import os
 import stat
-import string
 import time
 import types
 import portage_exception
-import portage_file
 import portage_util
 import portage_data
 from portage_localization import _
 
 HARDLINK_FD = -2
 
-hardlock_path_list = []
-def clean_my_hardlocks():
-	for x in hardlock_path_list:
-		hardlock_cleanup(x)
-def add_hardlock_file_to_cleanup(path):
-	mypath = portage_file.normpath(path)
-	if os.path.isfile(mypath):
-		mypath = os.path.dirname(mypath)
-	if os.path.isdir(mypath):
-		hardlock_path_list = mypath[:]
-
-atexit.register(clean_my_hardlocks)
-
 def lockdir(mydir):
 	return lockfile(mydir,wantnewlockfile=1)
 def unlockdir(mylock):
 	return unlockfile(mylock)
 
-def lockfile(mypath,wantnewlockfile=0,unlinkfile=0):
+def lockfile(mypath, wantnewlockfile=0, unlinkfile=0, waiting_msg=None):
 	"""Creates all dirs upto, the given dir. Creates a lockfile
 	for the given directory as the file: directoryname+'.portage_lockfile'."""
 	import fcntl
@@ -55,7 +39,9 @@ def lockfile(mypath,wantnewlockfile=0,unlinkfile=0):
 		wantnewlockfile = 0
 		unlinkfile      = 0
 	elif wantnewlockfile:
-		lockfilename = mypath+".portage_lockfile"
+		base, tail = os.path.split(mypath)
+		lockfilename = os.path.join(base, "." + tail + ".portage_lockfile")
+		del base, tail
 		unlinkfile   = 1
 	else:
 		lockfilename = mypath
@@ -69,11 +55,10 @@ def lockfile(mypath,wantnewlockfile=0,unlinkfile=0):
 			try:
 				if os.stat(lockfilename).st_gid != portage_data.portage_gid:
 					os.chown(lockfilename,os.getuid(),portage_data.portage_gid)
-			except SystemExit, e:
-				raise
 			except OSError, e:
 				if e[0] == 2: # No such file or directory
-					return lockfile(mypath,wantnewlockfile,unlinkfile)
+					return lockfile(mypath, wantnewlockfile=wantnewlockfile,
+						unlinkfile=unlinkfile, waiting_msg=waiting_msg)
 				else:
 					portage_util.writemsg("Cannot chown a lockfile. This could cause inconvenience later.\n");
 			os.umask(old_mask)
@@ -96,10 +81,13 @@ def lockfile(mypath,wantnewlockfile=0,unlinkfile=0):
 			raise
 		if e.errno == errno.EAGAIN:
 			# resource temp unavailable; eg, someone beat us to the lock.
-			if type(mypath) == types.IntType:
-				print "waiting for lock on fd %i" % myfd
-			else:
-				print "waiting for lock on %s" % lockfilename
+			if waiting_msg is None:
+				if isinstance(mypath, int):
+					print "waiting for lock on fd %i" % myfd
+				else:
+					print "waiting for lock on %s" % lockfilename
+			elif waiting_msg:
+				print waiting_msg
 			# try for the exclusive lock now.
 			fcntl.lockf(myfd,fcntl.LOCK_EX)
 		elif e.errno == errno.ENOLCK:
@@ -111,7 +99,7 @@ def lockfile(mypath,wantnewlockfile=0,unlinkfile=0):
 					try:
 						if os.stat(lockfilename)[stat.ST_NLINK] == 1:
 							os.unlink(lockfilename)
-					except Exception, e:
+					except OSError:
 						pass
 					link_success = hardlink_lockfile(lockfilename)
 			if not link_success:
@@ -122,11 +110,14 @@ def lockfile(mypath,wantnewlockfile=0,unlinkfile=0):
 			raise
 
 		
-	if type(lockfilename) == types.StringType and not os.path.exists(lockfilename):
+	if type(lockfilename) == types.StringType and \
+		myfd != HARDLINK_FD and os.fstat(myfd).st_nlink == 0:
 		# The file was deleted on us... Keep trying to make one...
 		os.close(myfd)
 		portage_util.writemsg("lockfile recurse\n",1)
-		lockfilename,myfd,unlinkfile,locking_method = lockfile(mypath,wantnewlockfile,unlinkfile)
+		lockfilename, myfd, unlinkfile, locking_method = lockfile(
+			mypath, wantnewlockfile=wantnewlockfile, unlinkfile=unlinkfile,
+			waiting_msg=waiting_msg)
 
 	portage_util.writemsg(str((lockfilename,myfd,unlinkfile))+"\n",1)
 	return (lockfilename,myfd,unlinkfile,locking_method)
@@ -147,20 +138,19 @@ def unlockfile(mytuple):
 		unhardlink_lockfile(lockfilename)
 		return True
 	
+	# myfd may be None here due to myfd = mypath in lockfile()
 	if type(lockfilename) == types.StringType and not os.path.exists(lockfilename):
 		portage_util.writemsg("lockfile does not exist '%s'\n" % lockfilename,1)
-		if (myfd != None) and type(lockfilename) == types.StringType:
+		if myfd is not None:
 			os.close(myfd)
 		return False
 
 	try:
-		if myfd == None:
+		if myfd is None:
 			myfd = os.open(lockfilename, os.O_WRONLY,0660)
 			unlinkfile = 1
 		locking_method(myfd,fcntl.LOCK_UN)
-	except SystemExit, e:
-		raise
-	except Exception, e:
+	except OSError:
 		if type(lockfilename) == types.StringType:
 			os.close(myfd)
 		raise IOError, "Failed to unlock file '%s'\n" % lockfilename
@@ -177,15 +167,15 @@ def unlockfile(mytuple):
 			# We won the lock, so there isn't competition for it.
 			# We can safely delete the file.
 			portage_util.writemsg("Got the lockfile...\n",1)
-			#portage_util.writemsg("Unlinking...\n")
-			os.unlink(lockfilename)
-			portage_util.writemsg("Unlinked lockfile...\n",1)
-			locking_method(myfd,fcntl.LOCK_UN)
-	except SystemExit, e:
-		raise
+			if os.fstat(myfd).st_nlink == 1:
+				os.unlink(lockfilename)
+				portage_util.writemsg("Unlinked lockfile...\n",1)
+				locking_method(myfd,fcntl.LOCK_UN)
+			else:
+				portage_util.writemsg("lockfile does not exist '%s'\n" % lockfilename,1)
+				os.close(myfd)
+				return False
 	except Exception, e:
-		# We really don't care... Someone else has the lock.
-		# So it is their problem now.
 		portage_util.writemsg("Failed to get lock... someone took it.\n",1)
 		portage_util.writemsg(str(e)+"\n",1)
 
@@ -203,28 +193,11 @@ def unlockfile(mytuple):
 def hardlock_name(path):
 	return path+".hardlock-"+os.uname()[1]+"-"+str(os.getpid())
 
-def hardlink_active(lock):
-	if not os.path.exists(lock):
-		return False
- 	# XXXXXXXXXXXXXXXXXXXXXXXXXX
-
 def hardlink_is_mine(link,lock):
 	try:
-		myhls = os.stat(link)
-		mylfs = os.stat(lock)
-	except SystemExit, e:
-		raise
-	except:
-		myhls = None
-		mylfs = None
-
-	if myhls:
-		if myhls[stat.ST_NLINK] == 2:
-			return True
-		if mylfs:
-			if mylfs[stat.ST_INO] == myhls[stat.ST_INO]:
-				return True
-	return False
+		return os.stat(link).st_nlink == 2
+	except OSError:
+		return False
 
 def hardlink_lockfile(lockfilename, max_wait=14400):
 	"""Does the NFS, hardlink shuffle to ensure locking on the disk.
@@ -234,9 +207,7 @@ def hardlink_lockfile(lockfilename, max_wait=14400):
 	Otherwise we lather, rise, and repeat.
 	We default to a 4 hour timeout.
 	"""
-	
-	add_hardlock_file_to_cleanup(lockfilename)
-	
+
 	start_time = time.time()
 	myhardlock = hardlock_name(lockfilename)
 	reported_waiting = False
@@ -251,11 +222,7 @@ def hardlink_lockfile(lockfilename, max_wait=14400):
 
 		try:
 			res = os.link(myhardlock, lockfilename)
-		except SystemExit, e:
-			raise
-		except Exception, e:
-			#print "lockfile(): Hardlink: Link failed."
-			#print "Exception: ",e
+		except OSError:
 			pass
 
 		if hardlink_is_mine(myhardlock, lockfilename):
@@ -268,10 +235,11 @@ def hardlink_lockfile(lockfilename, max_wait=14400):
 			portage_util.writemsg(".")
 		else:
 			reported_waiting = True
+			from portage_const import PORTAGE_BIN_PATH
 			print
 			print "Waiting on (hardlink) lockfile: (one '.' per 3 seconds)"
 			print "This is a feature to prevent distfiles corruption."
-			print "/usr/lib/portage/bin/clean_locks can fix stuck locks."
+			print "%s/clean_locks can fix stuck locks." % PORTAGE_BIN_PATH
 			print "Lockfile: " + lockfilename
 		time.sleep(3)
 	
@@ -280,15 +248,16 @@ def hardlink_lockfile(lockfilename, max_wait=14400):
 
 def unhardlink_lockfile(lockfilename):
 	myhardlock = hardlock_name(lockfilename)
-	try:
-		if os.path.exists(myhardlock):
-			os.unlink(myhardlock)
-		if os.path.exists(lockfilename):
+	if hardlink_is_mine(myhardlock, lockfilename):
+		# Make sure not to touch lockfilename unless we really have a lock.
+		try:
 			os.unlink(lockfilename)
-	except SystemExit, e:
-		raise
-	except:
-		portage_util.writemsg("Something strange happened to our hardlink locks.\n")
+		except OSError:
+			pass
+	try:
+		os.unlink(myhardlock)
+	except OSError:
+		pass
 
 def hardlock_cleanup(path, remove_all_locks=False):
 	mypid  = str(os.getpid())
@@ -301,11 +270,11 @@ def hardlock_cleanup(path, remove_all_locks=False):
 	mylist = {}
 	for x in mydl:
 		if os.path.isfile(path+"/"+x):
-			parts = string.split(x, ".hardlock-")
+			parts = x.split(".hardlock-")
 			if len(parts) == 2:
 				filename = parts[0]
-				hostpid  = string.split(parts[1],"-")
-				host  = string.join(hostpid[:-1], "-")
+				hostpid  = parts[1].split("-")
+				host  = "-".join(hostpid[:-1])
 				pid   = hostpid[-1]
 				
 				if not mylist.has_key(filename):
@@ -319,13 +288,13 @@ def hardlock_cleanup(path, remove_all_locks=False):
 
 	results.append("Found %(count)s locks" % {"count":mycount})
 	
-	for x in mylist.keys():
+	for x in mylist:
 		if mylist[x].has_key(myhost) or remove_all_locks:
 			mylockname = hardlock_name(path+"/"+x)
 			if hardlink_is_mine(mylockname, path+"/"+x) or \
 			   not os.path.exists(path+"/"+x) or \
 				 remove_all_locks:
-				for y in mylist[x].keys():
+				for y in mylist[x]:
 					for z in mylist[x][y]:
 						filename = path+"/"+x+".hardlock-"+y+"-"+z
 						if filename == mylockname:
@@ -334,26 +303,20 @@ def hardlock_cleanup(path, remove_all_locks=False):
 							# We're sweeping through, unlinking everyone's locks.
 							os.unlink(filename)
 							results.append(_("Unlinked: ") + filename)
-						except SystemExit, e:
-							raise
-						except Exception,e:
+						except OSError:
 							pass
 				try:
 					os.unlink(path+"/"+x)
 					results.append(_("Unlinked: ") + path+"/"+x)
 					os.unlink(mylockname)
 					results.append(_("Unlinked: ") + mylockname)
-				except SystemExit, e:
-					raise
-				except Exception,e:
+				except OSError:
 					pass
 			else:
 				try:
 					os.unlink(mylockname)
 					results.append(_("Unlinked: ") + mylockname)
-				except SystemExit, e:
-					raise
-				except Exception,e:
+				except OSError:
 					pass
 
 	return results
