@@ -109,11 +109,16 @@ class Manifest(object):
 			a Manifest (not needed for parsing and checking sums)."""
 		self.pkgdir = pkgdir.rstrip(os.sep) + os.sep
 		self.fhashdict = {}
-		self.hashes = portage_const.MANIFEST2_HASH_FUNCTIONS[:]
-		self.hashes.append("size")
+		self.hashes = set()
+		self.hashes.update(portage_const.MANIFEST2_HASH_FUNCTIONS)
 		if manifest1_compat:
-			self.hashes.extend(portage_const.MANIFEST1_HASH_FUNCTIONS)
-		self.hashes = sets.Set(self.hashes)
+			self.hashes.update(portage_const.MANIFEST1_HASH_FUNCTIONS)
+		self.hashes.difference_update(hashname for hashname in \
+			list(self.hashes) if hashname not in hashfunc_map)
+		self.hashes.add("size")
+		if manifest1_compat:
+			self.hashes.add(portage_const.MANIFEST1_REQUIRED_HASH)
+		self.hashes.add(portage_const.MANIFEST2_REQUIRED_HASH)
 		for t in portage_const.MANIFEST2_IDENTIFIERS:
 			self.fhashdict[t] = {}
 		self.compat = manifest1_compat
@@ -313,6 +318,7 @@ class Manifest(object):
 				myentry = Manifest2Entry(
 					type=t, name=f, hashes=self.fhashdict[t][f].copy())
 				myhashkeys = myentry.hashes.keys()
+				myhashkeys.sort()
 				for h in myhashkeys:
 					if h not in ["size"] + portage_const.MANIFEST2_HASH_FUNCTIONS:
 						del myentry.hashes[h]
@@ -329,11 +335,16 @@ class Manifest(object):
 		if self.compat:
 			cvp_list = self.fetchlist_dict.keys()
 			cvp_list.sort()
+			manifest1_hashes = set(hashname for hashname in \
+				portage_const.MANIFEST1_HASH_FUNCTIONS \
+				if hashname in hashfunc_map)
+			manifest1_hashes.add(portage_const.MANIFEST1_REQUIRED_HASH)
+			manifest1_hashes.add("size")
 			for cpv in cvp_list:
 				digest_path = os.path.join("files", "digest-%s" % self._catsplit(cpv)[1])
 				dname = os.path.join(self.pkgdir, digest_path)
 				try:
-					myhashes = perform_multiple_checksums(dname, portage_const.MANIFEST1_HASH_FUNCTIONS+["size"])
+					myhashes = perform_multiple_checksums(dname, manifest1_hashes)
 					myhashkeys = myhashes.keys()
 					myhashkeys.sort()
 					for h in myhashkeys:
@@ -443,11 +454,25 @@ class Manifest(object):
 		self.__init__(self.pkgdir, self.distdir,
 			fetchlist_dict=self.fetchlist_dict, from_scratch=True,
 			manifest1_compat=self.compat)
+		cpvlist = []
+		pn = os.path.basename(self.pkgdir.rstrip(os.path.sep))
+		cat = self._pkgdir_category()
 		for pkgdir, pkgdir_dirs, pkgdir_files in os.walk(self.pkgdir):
 			break
 		for f in pkgdir_files:
 			if f.endswith(".ebuild"):
 				mytype = "EBUILD"
+				pf = f[:-7]
+				ps = portage_versions.pkgsplit(pf)
+				cpv = "%s/%s" % (cat, pf)
+				if not ps:
+					raise portage_exception.PortagePackageException(
+						"Invalid package name: '%s'" % cpv)
+				if ps[0] != pn:
+					raise portage_exception.PortagePackageException(
+						"Package name does not " + \
+						"match directory name: '%s'" % cpv)
+				cpvlist.append(cpv)
 			elif manifest2MiscfileFilter(f):
 				mytype = "MISC"
 			else:
@@ -464,7 +489,6 @@ class Manifest(object):
 				continue
 			self.fhashdict["AUX"][f] = perform_multiple_checksums(
 				os.path.join(self.pkgdir, "files", f.lstrip(os.sep)), self.hashes)
-		cpvlist = [os.path.join(self._pkgdir_category(), x[:-7]) for x in os.listdir(self.pkgdir) if x.endswith(".ebuild")]
 		distlist = set()
 		for cpv in cpvlist:
 			distlist.update(self._getCpvDistfiles(cpv))
